/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/** Class to replace rect elements with invisible buttons */
function SvgPartsButtons(svgLayer, buttonsPlace, fktTitle) {
    this.node = buttonsPlace;
    this._btns = Array.from(svgLayer.childNodes)
        .filter(node => node.nodeType == Node.ELEMENT_NODE)
        .map((svgPart, index) => {
            const label = svgPart.hasAttribute('title') ? svgPart.getAttribute('title') : ('Bestandteil ' + (1 + index));
            const title = (fktTitle instanceof Function) ? fktTitle.call(this, label, svgPart, index) : '';
            const btn = buttonsPlace.ownerDocument.createElement('button');
            btn.textContent = label;
            if (title)
                btn.setAttribute('title', title);
            buttonsPlace.appendChild(btn);
            btn.addEventListener('click', ev => {
                ev.preventDefault();
                if (this.onclick instanceof Function)
                    this.onclick.call(this, label, svgPart, index);
            });
            btn.addEventListener('focus', ev => (this.onenter instanceof Function) && this.onenter.call(this, label, svgPart, index));
            btn.addEventListener('mousemove', ev => (this.onenter instanceof Function) && this.onenter.call(this, label, svgPart, index));
            return { svgPart, btn, label };
        });
    this.reposition();
    (new MutationObserver(() => this.reposition())).observe(buttonsPlace, { attributes: true, childList: false, subtree: false });
    window.addEventListener('resize', () => this.reposition());
}

SvgPartsButtons.prototype.onclick = function(label, svgPart, index) {}
SvgPartsButtons.prototype.onenter = function(label, svgPart, index) {}

SvgPartsButtons.prototype.reposition = function() {
    const parent = this.node.getBoundingClientRect();
    this._btns.forEach(({ svgPart, btn }) => {
        const part = svgPart.getBoundingClientRect();
        btn.style.width = part.width + 'px';
        btn.style.height = part.height + 'px';
        btn.style.left = (part.left - parent.left) + 'px';
        btn.style.top = (part.top - parent.top) + 'px';
    });
}

SvgPartsButtons.prototype.getPartByTitle = function(
    title       // Title of svg element in question
) {
    return this._btns.reduce((prev, { svgPart, label }) => prev || ((label == title) ? svgPart : null), null);
}
